@@ -1,6 +1,6 @@
 /* GNU Datamash - perform simple calculation on input data
 
-   Copyright (C) 2013-2016 Assaf Gordon <assafgordon@gmail.com>
+   Copyright (C) 2013-2017 Assaf Gordon <assafgordon@gmail.com>
 
    This file is part of GNU Datamash.
 
@@ -34,6 +34,7 @@
 
 #include "system.h"
 
+#include "die.h"
 #include "fpucw.h"
 #include "closeout.h"
 #include "hash.h"
@@ -147,7 +148,7 @@ group_columns_find_named_columns ()
 
       p->num = get_input_field_number (p->name);
       if (p->num == 0)
-        error (EXIT_FAILURE, 0,
+        die (EXIT_FAILURE, 0,
                 _("column name %s not found in input file"),
                 quote (p->name));
       p->by_name = false;
@@ -189,7 +190,7 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
       fputs ("  bin, strbin, round, floor, ceil, trunc, frac\n", stdout);
 
       fputs (_("Numeric Grouping operations:\n"),stdout);
-      fputs ("  sum, min, max, absmin, absmax\n",stdout);
+      fputs ("  sum, min, max, absmin, absmax, range\n",stdout);
 
       fputs (_("Textual/Numeric Grouping operations:\n"),stdout);
       fputs ("  count, first, last, rand, unique, collapse, countunique\n",
@@ -197,7 +198,7 @@ which require a pair of fields (e.g. 'pcov 2:6').\n"), stdout);
 
       fputs (_("Statistical Grouping operations:\n"),stdout);
       fputs ("\
-  mean, median, q1, q3, iqr, mode, antimode, pstdev, sstdev, pvar,\n\
+  mean, median, q1, q3, iqr, perc, mode, antimode, pstdev, sstdev, pvar,\n\
   svar, mad, madraw, pskew, sskew, pkurt, skurt, dpo, jarque,\n\
   scov, pcov, spearson, ppearson\n\
 \n", stdout);
@@ -353,7 +354,7 @@ process_line (const struct line_record_t *line)
           char *tmp = xmalloc (len+1);
           memcpy (tmp,str,len);
           tmp[len] = 0 ;
-          error (EXIT_FAILURE, 0,
+          die (EXIT_FAILURE, 0,
               _("%s in line %"PRIuMAX" field %"PRIuMAX": '%s'"),
               field_op_collect_result_name (flocr),
               (uintmax_t)line_number, (uintmax_t)op->field, tmp);
@@ -442,8 +443,14 @@ print_column_headers ()
 
       if (op->field > get_num_column_headers ())
         error_not_enough_fields (op->field, get_num_column_headers ());
-      printf ("%s" "(%s)",get_field_operation_name (op->op),
-                          get_input_field_name (op->field));
+
+      printf ("%s", get_field_operation_name (op->op));
+
+      if (op->op == OP_PERCENTILE) {
+        printf (":%lu", op->params.percentile);
+      }
+
+      printf ("(%s)", get_input_field_name (op->field));
 
       if (i != dm->num_ops-1)
         print_field_separator ();
@@ -463,7 +470,7 @@ field_op_find_named_columns ()
         continue;
       p->field = get_input_field_number (p->field_name);
       if (p->field == 0)
-        error (EXIT_FAILURE, 0,
+        die (EXIT_FAILURE, 0,
                 _("column name %s not found in input file"),
                 quote (p->field_name));
       p->field_by_name = false;
@@ -673,7 +680,7 @@ transpose_file ()
       const size_t num_fields = line_record_num_fields (thisline);
 
       if (strict && line_number>1 && num_fields != prev_num_fields)
-          error (EXIT_FAILURE, 0, _("transpose input error: line %"PRIuMAX" " \
+          die (EXIT_FAILURE, 0, _("transpose input error: line %"PRIuMAX" " \
                     "has %"PRIuMAX" fields (previous lines had %"PRIuMAX");\n" \
                     "see --help to disable strict mode"),
                     (uintmax_t)line_number, (uintmax_t)num_fields,
@@ -730,7 +737,7 @@ reverse_fields_in_file ()
       const size_t num_fields = line_record_num_fields (thisline);
 
       if (strict && line_number>1 && num_fields != prev_num_fields)
-          error (EXIT_FAILURE, 0, _("reverse-field input error: line " \
+          die (EXIT_FAILURE, 0, _("reverse-field input error: line " \
                        "%"PRIuMAX" has %"PRIuMAX" fields (previous lines had " \
                        "%"PRIuMAX");\n" \
                        "see --help to disable strict mode"),
@@ -818,6 +825,9 @@ tabular_check_file ()
   struct line_record_t lb1, lb2;
   struct line_record_t *thisline, *prevline;
 
+  const uintmax_t n_lines = dm->mode_params.check_params.n_lines;
+  const uintmax_t n_fields = dm->mode_params.check_params.n_fields;
+
   thisline = &lb1;
   prevline = &lb2;
 
@@ -832,7 +842,26 @@ tabular_check_file ()
 
       const size_t num_fields = line_record_num_fields (thisline);
 
-      if (line_number>1 && num_fields != prev_num_fields)
+      /* Check if the number of fields is different than expected/requested
+         on the command line (e.g. with 'datamash check 6 fields') */
+      if (n_fields && n_fields != num_fields)
+        {
+          fprintf (stderr, _("line %"PRIuMAX" (%"PRIuMAX" fields):\n  "),
+                       (uintmax_t)(line_number), (uintmax_t)num_fields);
+          ignore_value (fwrite (line_record_buffer (thisline),
+                                line_record_length (thisline), sizeof (char),
+                                stderr));
+          fputc ('\n', stderr);
+          die (EXIT_FAILURE, 0, _("check failed: line " \
+                       "%"PRIuMAX" has %"PRIuMAX" fields (expecting "\
+                       "%"PRIuMAX")"),
+                       (uintmax_t)line_number, (uintmax_t)num_fields,
+                       (uintmax_t)n_fields);
+        }
+
+      /* Check if the the number of fields changed from one line to the next
+         (only if no expected number of fields specified on the command line).*/
+      else if (line_number>1 && num_fields != prev_num_fields)
         {
           fprintf (stderr, _("line %"PRIuMAX" (%"PRIuMAX" fields):\n  "),
                        (uintmax_t)(line_number-1), (uintmax_t)prev_num_fields);
@@ -846,7 +875,7 @@ tabular_check_file ()
                                 line_record_length (thisline), sizeof (char),
                                 stderr));
           fputc ('\n', stderr);
-          error (EXIT_FAILURE, 0, _("check failed: line " \
+          die (EXIT_FAILURE, 0, _("check failed: line " \
                        "%"PRIuMAX" has %"PRIuMAX" fields (previous line had "\
                        "%"PRIuMAX")"),
                        (uintmax_t)line_number, (uintmax_t)num_fields,
@@ -857,6 +886,14 @@ tabular_check_file ()
       SWAP_LINES (prevline, thisline);
     }
 
+  /* Check if we read too many/few lines */
+  if (n_lines && n_lines != line_number)
+    {
+      die (EXIT_FAILURE, 0, _("check failed: input had %"PRIuMAX" lines " \
+                              "(expecting %"PRIuMAX")"),
+           (uintmax_t)line_number, (uintmax_t)n_lines);
+    }
+
   /* Print summary */
   printf (ngettext ("%"PRIuMAX" line", "%"PRIuMAX" lines",
           select_plural (line_number)), (uintmax_t)line_number);
@@ -962,7 +999,7 @@ remove_dups_in_file ()
        /* Add key to buffer (if not found) */
        const int i = hash_insert_if_absent (ht, next_key, NULL);
        if ( i == -1 )
-         error (EXIT_FAILURE, errno, _("hash memory allocation error"));
+         die (EXIT_FAILURE, errno, _("hash memory allocation error"));
 
        if ( i == 1 )
          {
@@ -1023,14 +1060,14 @@ open_input ()
           snprintf (tmp,sizeof (tmp),"-k%"PRIuMAX",%"PRIuMAX" ",
                     (uintmax_t)col_num,(uintmax_t)col_num);
           if (strlen (tmp)+strlen (cmd)+1 >= sizeof (cmd))
-            error (EXIT_FAILURE, 0,
+            die (EXIT_FAILURE, 0,
                    _("sort command too-long (please report this bug)"));
           strcat (cmd,tmp);
         }
 
       input_stream = popen (cmd,"r");
       if (input_stream == NULL)
-        error (EXIT_FAILURE, 0, _("failed to run 'sort': popen failed"));
+        die (EXIT_FAILURE, 0, _("failed to run 'sort': popen failed"));
     }
   else
     {
@@ -1046,7 +1083,7 @@ close_input ()
   int i;
 
   if (ferror (input_stream))
-    error (EXIT_FAILURE, errno, _("read error"));
+    die (EXIT_FAILURE, errno, _("read error"));
 
   if (pipe_through_sort)
     i = pclose (input_stream);
@@ -1054,7 +1091,7 @@ close_input ()
     i = fclose (input_stream);
 
   if (i != 0)
-    error (EXIT_FAILURE, errno, _("read error (on close)"));
+    die (EXIT_FAILURE, errno, _("read error (on close)"));
 }
 
 int main (int argc, char* argv[])
@@ -1129,7 +1166,7 @@ int main (int argc, char* argv[])
 
         case 't':
           if (optarg[0] == '\0' || optarg[1] != '\0')
-            error (EXIT_FAILURE, 0,
+            die (EXIT_FAILURE, 0,
                    _("the delimiter must be a single character"));
           in_tab = out_tab = optarg[0];
           break;
@@ -1180,7 +1217,7 @@ int main (int argc, char* argv[])
 
   /* If using named-columns, but no input header - abort */
   if (dm->header_required && !input_header)
-    error (EXIT_FAILURE, 0,
+    die (EXIT_FAILURE, 0,
            _("-H or --header-in must be used with named columns"));
 
   open_input ();
@@ -1223,8 +1260,8 @@ int main (int argc, char* argv[])
       tabular_check_file ();
       break;
 
-    case MODE_INVALID:
-    default:
+    case MODE_INVALID:                           /* LCOV_EXCL_LINE */
+    default:                                     /* LCOV_EXCL_LINE */
       internal_error ("op mode");                /* LCOV_EXCL_LINE */
     }
   free_column_headers ();
